<?php

namespace Forms\Leave\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Forms\Leave\Http\Requests\LeaveRequest;
use Forms\Leave\Interfaces\LeaveRepositoryInterface;
use Forms\Leave\Models\Leave;

/**
 * Resource controller class for leave.
 */
class LeaveResourceController extends BaseController
{

    /**
     * Initialize leave resource controller.
     *
     * @param type LeaveRepositoryInterface $leave
     *
     * @return null
     */
    public function __construct(LeaveRepositoryInterface $leave)
    {
        parent::__construct();
        $this->repository = $leave;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Forms\Leave\Repositories\Criteria\LeaveResourceCriteria::class);
    }

    /**
     * Display a list of leave.
     *
     * @return Response
     */
    public function index(LeaveRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Forms\Leave\Repositories\Presenter\LeavePresenter::class)
                ->$function();
        }

        $leaves = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('leave::leave.names'))
            ->view('leave::leave.index', true)
            ->data(compact('leaves', 'view'))
            ->output();
    }

    /**
     * Display leave.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function show(LeaveRequest $request, Leave $leave)
    {

        if ($leave->exists) {
            $view = 'leave::leave.show';
        } else {
            $view = 'leave::leave.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('leave::leave.name'))
            ->data(compact('leave'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new leave.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LeaveRequest $request)
    {

        $leave = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('leave::leave.name')) 
            ->view('leave::leave.create', true) 
            ->data(compact('leave'))
            ->output();
    }

    /**
     * Create new leave.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LeaveRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $leave                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('leave::leave.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('leave/leave/' . $leave->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/leave/leave'))
                ->redirect();
        }

    }

    /**
     * Show leave for editing.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function edit(LeaveRequest $request, Leave $leave)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('leave::leave.name'))
            ->view('leave::leave.edit', true)
            ->data(compact('leave'))
            ->output();
    }

    /**
     * Update the leave.
     *
     * @param Request $request
     * @param Model   $leave
     *
     * @return Response
     */
    public function update(LeaveRequest $request, Leave $leave)
    {
        try {
            $attributes = $request->all();

            $leave->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('leave::leave.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('leave/leave/' . $leave->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('leave/leave/' . $leave->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the leave.
     *
     * @param Model   $leave
     *
     * @return Response
     */
    public function destroy(LeaveRequest $request, Leave $leave)
    {
        try {

            $leave->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('leave::leave.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('leave/leave/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('leave/leave/' . $leave->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple leave.
     *
     * @param Model   $leave
     *
     * @return Response
     */
    public function delete(LeaveRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('leave::leave.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('leave/leave'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/leave/leave'))
                ->redirect();
        }

    }

    /**
     * Restore deleted leaves.
     *
     * @param Model   $leave
     *
     * @return Response
     */
    public function restore(LeaveRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('leave::leave.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/leave/leave'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/leave/leave/'))
                ->redirect();
        }

    }

}
